package org.example;
import java.util.ArrayList;
import java.util.List;
public class ThreadPool {
    private MyBlockingQueue<Runnable> taskQueue;
    private List<PoolThreadRunnable> runnables;
    private boolean isStopped;
    private int completedTasks;
    public ThreadPool(int noOfThreads, int maxNoOfTasks) {
        taskQueue = new MyBlockingQueue<>(maxNoOfTasks);
        runnables = new ArrayList<>();
        isStopped = false;


        for (int i = 0; i < noOfThreads; i++) {
            PoolThreadRunnable poolThreadRunnable = new PoolThreadRunnable(taskQueue);
            runnables.add(poolThreadRunnable);
            new Thread(poolThreadRunnable).start();
        }
    }
    public synchronized void execute(Runnable task) throws Exception {
        if (isStopped) {
            throw new IllegalStateException("ThreadPool is stopped");
        }
        taskQueue.put(task);
    }
    public synchronized void stop() {
        this.isStopped = true;
        for (PoolThreadRunnable runnable : runnables) {
            runnable.doStop();
        }
    }
    public void waitUntilAllTasksFinished(long timeout ) {
        long startTime = System.currentTimeMillis();
        while(this.taskQueue.getQueueSize()>0) {
            if (System.currentTimeMillis() - startTime > timeout) {
                stop(); // Stop the ThreadPool if timeout is finished
                break;
            }
            try {
                Thread.sleep(1);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }

}